#!/usr/bin/env python
#! -*- coding: utf-8 -*-

from gi.repository import Clutter, GLib, ClutterGst, GObject, Gst
import sys, os
import json
from items import ItemFactory
from slide import Slide

class Presenter:
    def __init__(self):
        self.stage = stage = Clutter.Stage.get_default()
        stage.set_user_resizable(True)
        stage.connect("destroy", Clutter.main_quit)
        stage.set_color(Clutter.Color.new(0, 0, 0, 255))

        stage.connect('button-press-event', self.on_stage_button_press)
        stage.connect('key-press-event', self.on_stage_key_press)
        stage.connect('notify::allocation', self.on_stage_allocation)

        # Create top level container
        self.container = Clutter.ScrollActor.new()
        self.stage.add_child(self.container)
        self.container.set_easing_duration(200)
        self.container.set_easing_mode(Clutter.AnimationMode.LINEAR)

        self.slide_size = (1024, 768)
        self.slide_spacing = 0
        self.resources = os.path.join(os.path.abspath(sys.path[0]), 'resources')

        self.slides = []
        self.slide_no = 0
        self.title = 'Presention'
        self.item_factory = None
        self.edit_notification = None
        self.editing = False
        self.insertingText = False
        self.zoomed = False

        stage.set_title(self.title)

        self.create_controls()
        stage.set_fullscreen(True)

    def get_slide_size(self):
        return self.slide_size

    def update_slide_positions(self, duration=200):
        # Put slides in position on the stage
	self.container.save_easing_state()
	self.container.set_easing_duration(duration)

        # Calculate the x offset for each slide, based on the
        # stage width/height
        x_scale = min(self.stage_size[0] / self.slide_size[0], 1.0)
        y_scale = min(self.stage_size[1] / self.slide_size[1], 1.0)

        scale = min(x_scale, y_scale)
        self.slide_spacing = x_spacing = (self.stage_size[0] + 10) / scale
        x_offset = max(0, x_spacing - self.slide_size[0]) / 2
        """print "Scaling slides by %g to fit %d,%d screen spacing %d offset %g, %g" % \
             (scale, self.stage_size[0], self.stage_size[1], x_spacing, x_offset, y_offset)"""
	if self.zoomed:
            scale = min(self.stage_size[0] / (min(4, len(self.slides)) * self.slide_spacing), x_scale)

        self.container.set_size(x_spacing * len(self.slides), self.stage_size[1] / scale)

        y_offset = max(0, (self.stage_size[1] / scale) - self.slide_size[1]) / 2

        self.showSlide(self.slide_no)

        for i in range(len(self.slides)):
            x = x_spacing * i + x_offset
            self.slides[i].set_position(x, y_offset)

        self.container.set_scale(scale, scale)
	self.container.restore_easing_state()

    def on_stage_allocation(self, stage, allocation):
        box = stage.get_allocation_box()
        x1, y1, x2, y2 = \
            box.x1, box.y1, box.x2, box.y2

        self.center = ((x1 + x2) / 2, (y1 + y2) / 2)
        self.stage_size = (x2-x1, y2-y1)

        self.update_slide_positions(0)
        self.update_edit_notification()
        return False
    def update_edit_notification(self):
        if self.edit_notification:
            t_w = self.edit_notification.get_size()[0]
            self.edit_notification.set_position(self.stage_size[0]-t_w, 0)

    def create_controls(self):
        box = self.stage.get_allocation_box()
        itemfac = self.getItemFactory()

        # Create editing notification
        font = "Sans 16"
        col = Clutter.Color.new(0,128,0,255)
        self.edit_notification = Clutter.Text.new_full(font, "Editing", col)
        t_w, t_h = self.edit_notification.get_size()
        self.edit_notification.set_position(box.x2-t_w, 0)
        self.stage.add_child(self.edit_notification)
        self.edit_notification.hide()

    def present(self, slide_no = 0):
        if len(self.slides) == 0:
            self.newSlide()
        self.showSlide(slide_no)
	#self.update_slide_positions(0)

        self.stage.show()

    def load(self, filename):
        # print "Loading presentation"
        self.slides = []
        self.presentation_file = filename
        f = open(self.presentation_file, 'r')
	c = f.read()
	if c[0:2] == "#!":
	  l = c.split("\n")
	  c = "\n".join(l[1:])
        j = json.loads(c)
        self.deserialise(j)
        self.stage.set_title(self.title)

    def set_save_path(self, filename):
        self.presentation_file = filename

    def save(self, fname = None):
        print "Saving presentation"
        j = json.dumps(self.serialise(), sort_keys=True,
                indent=2, separators=(',', ': '))
        if fname:
            self.presentation_file = fname
        f = open(self.presentation_file, "w")
	f.write("#!/usr/bin/python ./present/presenter.py\n")
        f.write(j)

    def serialise(self):
        return { 'title': self.title, \
                 'slide-size' : self.slide_size, \
                 'slides' : map(lambda x : x.serialise(),self.slides) \
               }

    def loadSlide(self, j, itemfact):
	# print "Loading slide", j
	s = Slide(self, j, itemfact)
        s.set_parent(self.container)
	return s

    def deserialise(self, j):
        self.title = j["title"]
        i = self.getItemFactory()
        self.slides = map(lambda x : self.loadSlide(x, i), j['slides'])
        #for item in self.items:
                #self.items_by_id[item.iid] = item
    def getItemFactory(self):
        if self.item_factory is None:
            self.item_factory = ItemFactory(self)
        return self.item_factory

    def on_stage_button_press(self, stage, event):
        #print "Stage clicked at (%f, %f)" % (event.x, event.y)
	return True

    def on_stage_key_press(self, stage, event):
        code = event.keyval
        if code == Clutter.KEY_Q:
           Clutter.main_quit()
	elif code == Clutter.KEY_q:
    	   self.save()
           Clutter.main_quit()
	elif code == Clutter.KEY_S:
    	   self.save()
        elif code == Clutter.KEY_f:
           self.stage.set_fullscreen(not self.stage.get_fullscreen())
        elif code == Clutter.KEY_e:
           self.start_edit()
        elif code == Clutter.KEY_Escape:
	   if self.insertingText:
               self.insertingText = False
           else:
	       self.stop_edit()
        elif code == Clutter.KEY_space or code == Clutter.KEY_Right:
           self.nextSlide()
        elif code == Clutter.KEY_Left:
           self.prevSlide()
        elif code == Clutter.KEY_z:
           self.setZoom(not self.zoomed)
	elif self.editing:
	    if code == Clutter.KEY_n:
                self.newSlide(self.slide_no+1)
            elif code == Clutter.KEY_t:
                self.insertingText = True
	return True

    def on_slide_pressed(self, slide, event):
	if self.insertingText:
	    fac = self.getItemFactory()
	    t = fac.newTextItem()
	    slide.takeItem(t)
	    x, y = slide.transform_stage_point(event.x, event.y)
	    t.set_position(x, y)
	    # FIXME: Start editing the new text box
	    self.insertingText = False

    def get_max_id(self):
        max_id = 0
        for s in self.slides:
            max_id = max(max_id, s.get_max_id())
        return max_id

    def start_edit(self):
        if self.editing:
            return;
        # Show editing sign
        self.edit_notification.show()
        self.editing = True
	try:
	    self.slides[self.slide_no].start_edit()
	except:
	    pass

    def stop_edit(self):
        if not self.editing:
            return
        map(lambda x: x.stop_edit(), self.slides)

        # Hide editing sign
        self.edit_notification.hide()
	self.stage.set_key_focus(None)
        self.editing = False

    def newSlide(self, index=-1):
        s = Slide(self)
        s.set_parent(self.container)
	if index < 0:
	    index = len(self.slides)
        self.slides.insert(index, s)
        self.update_slide_positions(0)
        self.showSlide(index)

    def showSlide(self, slide_no):
        if slide_no < 0 or slide_no >= len(self.slides):        
            return

        self.slide_no = slide_no
        self.slides[slide_no].show()
        point = Clutter.Point()
        point.x = self.slide_no * self.slide_spacing
        point.y = 0
        # print "Scroll to", point.x, point.y, "slide", slide_no
        self.container.scroll_to_point(point)
        self.edit_notification.set_text("Editing\nSlide %d" % (slide_no))
        self.update_edit_notification()
	
    def nextSlide(self):
        self.showSlide(self.slide_no + 1)
    def prevSlide(self):
        self.showSlide(self.slide_no - 1)
    def setZoom(self, zoomed):
	self.zoomed = zoomed
	if self.zoomed:
            map(lambda x: x.show(), self.slides)
	self.update_slide_positions()
    def load_asset(self, slide_no, path, itemfactory = None):
        if itemfactory is None:
           itemfactory = self.getItemFactory()
        item = itemfactory.loadFile(path)
        print "loaded", path
        self.slides[slide_no].takeItem(item)
    def dump_assets(self):
	for slide in self.slides:
	    slide.dump_assets()

if __name__ == '__main__':
    GObject.threads_init()
    Clutter.init(sys.argv)
    ClutterGst.init(sys.argv)
    Gst.init(sys.argv)

    if len(sys.argv) < 2:
        print "%s <presentation_file> [slide_no]" % (sys.argv[0])
        sys.exit(1)
    p = Presenter()
    if os.path.exists(sys.argv[1]):
        p.load(sys.argv[1])
    p.set_save_path(sys.argv[1])
    if len(sys.argv) > 2:
        p.present(int(sys.argv[2]))
    else:
        p.present()

    Clutter.main()
