#!/usr/bin/env python
#! -*- coding: utf-8 -*-

from gi.repository import Clutter, Cogl, GdkPixbuf
import os, re, sys
import items
import random

class Slide:
    def __init__(self, presentation, attrs = {}, itemfactory = None):
        vals = { 'items' : [],
		 'background-colour' : (10, 10, 10, 255) }
        vals.update(attrs)

        self.presentation = presentation
	self.items = [ ]
	self.background_colour = vals['background-colour']
	self.editing = False
        self.current = None

        self.content = Clutter.Actor.new() 
        self.content.set_size (*self.presentation.get_slide_size())
	c = Clutter.Color.new(*self.background_colour)
        self.content.set_background_color(c)
        self.content.set_easing_duration(200)
        self.content.set_easing_mode(Clutter.AnimationMode.LINEAR)
	self.content.set_reactive(True)

        self.content.connect('button-press-event', self.on_button_press)

        if itemfactory:
            map(lambda x : self.loadItem(x, itemfactory), vals['items'])
    def dump_assets(self):
	for i in self.items:
	    i.dump_assets()

    def loadItem(self, data, itemfac):
        i = itemfac.loadJSON(data)
        self.takeItem(i)

    def serialise(self):
        return { 'type' : 'Slide', \
            'items' : map(lambda x : x.serialise(),self.items),
	    'background-colour' : self.background_colour
	}
    def set_position(self, x, y):
	self.content.save_easing_state()
	self.content.set_easing_duration(0)
	self.content.set_position(x, y)
	self.content.restore_easing_state()
    def set_parent(self, parent):
	parent.add_child(self.content)
    def add_child(self, child):
	self.content.add_child(child)
    def transform_stage_point(self, stagex, stagey):
	res, x, y = self.content.transform_stage_point(stagex, stagey)
	return (x,y)
    def show(self):
        map(lambda x: x.show(), self.items)
	self.content.show()
    def hide(self):
	self.content.hide()
    def takeItem(self, item):
        item.unparent()
	self.items.append(item)
	item.reparent(self, self.content)
	if self.editing:
	    item.start_edit()
    def on_button_press(self, actor, event):
        return self.presentation.on_slide_pressed(self, event)

    def expand_item(self, item):
        if self.current is not None:
            self.current.unexpand()
        self.current = item
        self.raise_actor(self.current.actor)
        w, h = self.content.get_size()
        self.current.expand(0, 0, w, h)
    def start_edit(self):
	if self.editing:
	    return False
	self.editing = True
        map(lambda x: x.start_edit(), self.items)
    def stop_edit(self):
	if not self.editing:
	    return False
        map(lambda x: x.stop_edit(), self.items)
	self.editing = False
    def raise_actor(self, actor):
        p = actor.get_parent()
        p.set_child_above_sibling(actor, None)
