#!/usr/bin/env python

import sys, re, os
import gi
gi.require_version('Gst', '1.0')
from gi.repository import GObject, Clutter, ClutterGst, Gst

class VideoPlayer():
    def __init__(self):
        self.actor = Clutter.Actor.new()
        self.actor.set_reactive(True)
        # actor.connect('button-press-event', self.on_button_press)
        self.actor.connect('key-press-event', self.on_key_press)
        self.texture = Clutter.Texture.new()

        self.sink = ClutterGst.VideoSink.new(self.texture)
        self.actor.add_child(self.texture)
        self.player = Gst.ElementFactory.make("playbin", "playbin0")
        self.player.set_property("video-sink", self.sink)

	self.parent = None
        self.uri = None
        self.playing = False
        self.actor.set_size(640, 480)
	self.scale = 1.0

    def load(self, uri):
        if self.uri is not None:
            self.stop()
        if re.match("^.*://", uri):
            self.uri = uri
	else:
	    self.uri = "file://%s" % (os.path.abspath(uri))
        self.player.set_property("uri", self.uri)
	self.video_size = None
	print "URI", self.uri

    def preroll(self):
        self.player.set_state(Gst.State.PLAYING)
        self.player.get_state(5000000000)
        # self.player.set_state(Gst.State.PAUSED)
        pad = self.sink.get_static_pad("sink")
        caps = pad.get_current_caps()
        s = caps.get_structure(0)
        w = s.get_int("width")[1]
        h = s.get_int("height")[1]
        par = s.get_fraction("pixel-aspect-ratio")[1:]

        h *= par[0]
        h /= par[1]
        self.video_size = (w,h)
        self.texture.set_size(*self.video_size)
        self.actor.set_size(*self.video_size)
	print "prerolled. Vid size", self.video_size

    def play(self):
        if not self.playing:
          print "Starting..."
          self.player.set_state(Gst.State.PLAYING)
          self.playing = True
    def pause(self):
        print "Pausing..."
        if self.playing:
          self.player.set_state(Gst.State.PAUSED)
          self.playing = False;
    def stop(self):
        print "Stopping..."
        self.player.set_state(Gst.State.NULL)
        self.playing = False
    def reparent(self, parent):
        if self.parent != parent:
            self.pause()
            if self.parent is not None:
                self.parent.remove_child(self.actor)
            if parent is not None:
                parent.add_child(self.actor)
            self.parent = parent
    def set_easing_duration(self, duration):
	self.actor.set_easing_duration(duration)
    def set_opacity(self, opacity):
	self.actor.set_opacity(opacity)
    def show(self):
        self.actor.show_all()
    def hide(self):
        self.actor.hide()
    def get_size(self):
        if self.video_size is None:
            self.preroll()
        return self.video_size
    def set_position(self, x, y):
        self.actor.set_position(x, y)
    def get_visible_size(self):
	w, h = self.get_size()
	w *= self.scale
	h *= self.scale
	return (w, h)
    def set_scaled_size(self, w, h):
        if self.video_size is None:
            self.preroll()
        w_scale = w / self.video_size[0]
        h_scale = h / self.video_size[1]
        self.scale = scale = min(h_scale, w_scale)
        self.actor.set_scale(scale, scale)

    def on_key_press(self, stage, event):
        code = event.keyval
        #print "Got key press for item", self.iid
        if code == Clutter.KEY_space:
           if self.playing:
             self.pause()
           else:
             self.play()
           return True
        return False

if __name__ == '__main__':
    GObject.threads_init()
    Clutter.init(sys.argv)
    Gst.init(sys.argv)

    if len(sys.argv) < 2:
        print "%s <video_file>" % (sys.argv[0])
        sys.exit(1)

    stage = Clutter.Stage.get_default()
    stage.connect("destroy", Clutter.main_quit)
    v = VideoPlayer()
    v.load(sys.argv[1])
    v.reparent(stage)
    v.play()
    stage.show_all()
